package com.company.timezonemysqlbug

import com.company.timezonemysqlbug.entity.TestLocalDateEntity
import io.jmix.core.Id
import io.jmix.core.security.SystemAuthenticator
import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.boot.test.util.TestPropertyValues
import org.springframework.context.ApplicationContextInitializer
import org.springframework.context.ConfigurableApplicationContext
import org.springframework.test.context.ContextConfiguration
import java.time.LocalDate
import java.time.ZoneId
import java.util.*

@SpringBootTest
@ContextConfiguration(initializers = [TimezoneMysqlBugApplicationTests.Initializer::class])
internal class TimezoneMysqlBugApplicationTests {

    companion object {
        private val testDatabaseContainer: TestDatabaseContainer = TestDatabaseContainer()
        @Volatile private var initialized = false
    }

    @Autowired protected lateinit var dataManager: io.jmix.core.DataManager
    @Autowired protected lateinit var systemAuthenticator: SystemAuthenticator

    class Initializer : ApplicationContextInitializer<ConfigurableApplicationContext> {
        override fun initialize(applicationContext: ConfigurableApplicationContext) {
            TestPropertyValues.of(
                "main.datasource.url=" + testDatabaseContainer.jdbcUrl,
                "main.datasource.username=" + testDatabaseContainer.username,
                "main.datasource.password=" + testDatabaseContainer.password
            ).applyTo(applicationContext.environment)
        }
    }

    @BeforeEach
    protected open fun setUpBeforeEach() {
        // test will pass if time zone is set to Europe/Vienna
        TimeZone.setDefault(TimeZone.getTimeZone(ZoneId.of("UTC")))
        if (!initialized) {
            initialized = true
        }
    }

    @Test
    fun testLocalDateMysqlIssue() {
        systemAuthenticator.withSystem {
            val initialEntity = dataManager.create(TestLocalDateEntity::class.java).let {
                it.name = "Test1"
                it.localDate = LocalDate.of(2022, 2, 1)
                dataManager.save(it)
            }
            assertThat(initialEntity.localDate)
                .isEqualTo("2022-02-01")

            val reloadedEntity = dataManager.load(Id.of(initialEntity)).one()
            assertThat(reloadedEntity.localDate)
                .isEqualTo("2022-02-01")
        }
    }
}